// koggtag (C)2001 Dario Abatianni <eisfuchs@tigress.com>
// This program is distributed under the GNU General Public Licence (GPL)

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <qstring.h>

#include <vorbis/vorbisfile.h>

#include "oggfile.h"
#include "tag.h"
#include "vcedit.h"

OggFile::OggFile()
{
  /* Create new filename and tag objects */
  filename=new QString();
  tags=new QList<Tag>();

  /* There was no data loaded yet */
  setOpen(false);
}

OggFile::~OggFile()
{
  /* Destroy the tag list object */
  delete tags;
  /* Destroy the File name object */
  delete filename;
}

void OggFile::load()
{
  printf("OggFile::load(): Opening file '%s'\n",getFilename()->latin1());

  OggVorbis_File file;

  int success=0;

  FILE *handle=fopen(getFilename()->latin1(),"r");
  if(handle==0) printf("OggFile::load(): fopen('%s')==0!\n",getFilename()->latin1());
  else
  {
    if(ov_open(handle,&file,NULL,0))
    {
      printf("OggFile::load(): ov_open('%s')!=0!\n",getFilename()->latin1());

      fclose(handle); /* only in case of failure. otherwise use ov_clear() to close! */
    }
    else
    {
      struct vorbis_comment *comment;

      comment=ov_comment(&file,-1);
      if(comment==0) printf("OggFile::load(): ov_comment()==0!\n");
      else
      {
        int tagnum=0;
        char **taglist=comment->user_comments;

        while(taglist[tagnum])
        {
          tags->append(new Tag(taglist[tagnum]) );

          printf("OggFile::load() %s\n",taglist[tagnum]);

          tagnum++;
        }
        success=1;
      }
      ov_clear(&file); /* ogg vorbis library fclose()s the file for us */
    }
  }

  setOpen(success);
}

void OggFile::unload()
{
  printf("OggFile::unload()\n");
  /* Remove all tags from Ogg File */
  while(tags->removeFirst());
  /* Declare file as closed, i.e. no data loaded */
  setOpen(false);
}

void OggFile::load(const char *openname)
{
  printf("OggFile::load(\"%s\")\n",openname);
  /* Set the filename for this Ogg File */
  setFilename(openname);
  /* Open it */
  load();
}

void OggFile::setOpen(bool open)
{
  /* Set the open flag according to the parameter */
  is_open=open;
}

bool OggFile::isOpen()
{
  return is_open;
}

void OggFile::write(QString *new_file_name)
{
  int dorename=0;
  char tmpname[14];

  FILE *handle=fopen(getFilename()->latin1(),"r");
  if(handle==0) printf("OggFile::write(): fopen('%s')==0!\n",getFilename()->latin1());
  else
  {
    vcedit_state *state=vcedit_new_state();
    if(vcedit_open(state,handle)==-1) printf("OggFile::write(): vcedit_open('%s')==-1!\n",getFilename()->latin1());
    else
    {
      QString *vendor=new QString();

      struct vorbis_comment *oc=vcedit_comments(state);
      if(oc)
      {
        vendor->setLatin1(oc->vendor);
      }

      struct vorbis_comment *vc=state->vc;
      if(vc) vorbis_comment_clear(vc);

      vc->vendor=(char *)calloc(1,vendor->length()+1);
      strcpy(vc->vendor,vendor->latin1());

      Tag *node=tags->first();
      while(node)
      {
        char *name;
        char *value;

        name=(char *) calloc(1,node->getName()->length()+1);
        value=(char *) calloc(1,node->getValue()->length()+1);

        strcpy(name,node->getName()->latin1());
        strcpy(value,node->getValue()->latin1());

        printf("OggFile::write(): Tag %s=%s\n",name,value);

        vorbis_comment_add_tag(vc,name,value);

        free(value);
        free(name);

        node=tags->next();
      }

      strcpy(tmpname,"koggfileXXXXXX");

      int tmphandle=mkstemp(tmpname);
      if(tmphandle==-1) printf("OggFile::write(): tmphandle==-1!\n");
      else
      {
        close(tmphandle);

        FILE *outfile=fopen(tmpname,"w");
        if(outfile==0)
        {
          printf("OggFile::write(): outfile==0!\n");
          vcedit_clear(state); // frees comment entries and vendor
        }
        else
        {
          vcedit_write(state,outfile); // calls vcedit_clear() itself so we don't free anything
        }

        fclose(outfile);
        dorename=1;
      }

      free(vc->vendor);
    }
    fclose(handle);
  }

  if(dorename)
  {
    rename(tmpname,(new_file_name) ? new_file_name->latin1() : getFilename()->latin1() );
  }
}

QString *OggFile::getFilename()
{
  return filename;
}

void OggFile::setFilename(const char *newname)
{
  /* Set the new file name of this Ogg File */
  filename->setLatin1(newname);
}
